* -------------------------------
* 	LPS Example
*
* 	Routine 3
*
*	Language: MC68K-ASM1
*	Style: plain MC68000
* 	Version: LPS20-L20-1-m68k
* -------------------------------

* Lecture L20 Plain MC68000 Solution 1

*  Questo esempio illustra la traduzione in ASM di un programma C89
;	che contiene un prototipo e in cui avvengono conversioni
;	di tipo nella chiamata di una funzione.
*  Vengono realizzate 2 traduzioni:
;	1) traduzione corretta, grazie alla presenza del prototipo
;	2) traduzione del programma C89 da cui viene rimosso il prototipo:
;		si usa la regola della dichiarazione implicita, che in
;		questo caso provoca un undefined behavior in quanto induce 
;		il traduttore ad applicare delle conversioni di dato errate

* Questo esempio e' realizzato per l'assembler ASM-One V1.48

* Variabili
; il tipo int viene memorizzato con word
; il tipo long viene memorizzato con long
	org	$4000

; la long seguente contiene il valore che la versione Plain C
; legge da standard input
x:	dc.l	0

; la long seguente, al termine del programma, contiene il valore
; che la versione Plain C stampa su standard output
z:	dc.l	0

; la word seguente contiene la variabile y
y:	dc.w	-99

* Programma
	org	$2000

inizio_codice:

* Routine main
main:
	* input
	; la lettura mediante scanf della versione Plain C
	; viene simulata dalla lettura della long a indirizzo x
	move.l	x,d4
	
	* uso dei registri (routine main, fase di calcolo)
	;	d4 (formato long) - variabile x
	;	d5 (formato long) - variabile z
	move.l	d4,d1	; argomento a
	move.w	y,d2	; copia y in d2
	ext.l	d2	; converto da int a long
	bsr	f
	; ora d0 in formato word contiene il risultato di f
	move.w	d0,d5	; copio in d5
	ext.l	d5	; converto da int a long

	* output
	; la scrittura mediante printf della versione Plain C
	; viene simulata dalla scrittura della long a indirizzo z
	move.l	d5,z

fine_main:

	* fine programma
	bra	fine_codice

* Routine f
; Input
;	d1 formato long - parametro a
;	d2 formato long - parametro b
; Output
;	d0 formato word - risultato
; Registri modificati
f:
	add.l	d2,d1
	divs	#1000,d1
	move.w	d1,d0
	rts

fine_codice:

	end

* Original Plain C Code

#include <stdio.h>

/* Prototipo */
int f( long a, long b );

/* Main */
int main( void ) {

	long x, z;
	int y = -99;

	scanf( "%ld", &x );

	z = f( x, y );
	
	printf( "%ld", z );

	return 0;
}

/* Definizione di f */
int f( long a, long b ) {
	a += b;
	return a / 1000;
}

/*
	Test Cases, prototipo presente (programma corretto)

	Test 1
	input: 100005
	output: 99

	Test 2
	input: -2000
	output: -2

	Test 3
	input: -2901
	output: -3

	Test 4
	input: 99999
	output: 99
*/

/*
	Test Cases, prototipo assente (programma non corretto)

	Test 1
	input: 100005
	output: 4295067

	Test 2
	input: -2000
	output: 4294965

	Test 3
	input: -2901
	output: 4294964

	Test 4
	input: 99999
	output: 4295067
*/
